;; Generated package description from sql-beeline.el  -*- no-byte-compile: t -*-
(define-package "sql-beeline" "0.2" "Beeline support for sql.el" 'nil :commit "b7ffff9404d93f569e3c2bc59db1be6e848c894f" :url "https://elpa.gnu.org/packages/sql-beeline.html" :authors '(("Filipp Gunbin" . "fgunbin@fastmail.fm")) :maintainer '("Filipp Gunbin" . "fgunbin@fastmail.fm") :keywords '("sql" "hive" "beeline" "hiveserver2" "impala"))
